#include "Adafruit_WS2801.h"
#include "SPI.h"
#ifdef __AVR_ATtiny85__
#include <avr/power.h>
#endif
#include "HX711.h"
#include <SoftwareSerial.h>
#include <DFRobotDFPlayerMini.h>

/* ARDUINO INPUT AND OUTPUT PINS */
#define RX = 0; // reserved for speaker
#define TX = 1; // reserved for speaker
uint8_t dataPin  = 2;    // Yellow wire on Adafruit Pixels // GREEN WIRE ON CHEAP STRING SET
uint8_t clockPin = 3;    // Green wire on Adafruit Pixels  // WHITE WIRE ON CHEAP STRING SET //may need to be 3 or 9???
#define CLK  5  //  WHITE WIRE Arduino pin 5 connect to HX711 CLK
#define DOUT  6  // YELLOW WIRE Arduino pin 6 connect to HX711 DOUT
#define BLACK_LIGHT_1 7
#define BLACK_LIGHT_2 8
#define BLACK_LIGHT_3 9
#define BUTTON_LEFT 10
#define BUTTON_FRONT 11
#define BUTTON_RIGHT 12
#define LASER_1 13
#define LASER_2 14
#define HALL_BACK 15
#define HALL_FRONT 16
#define RELAY_LOCK 17
#define LASER_SENSOR_1 A1
#define LASER_SENSOR_2 A2
#define SENSOR_1 A3
#define SENSOR_2 A4
#define SENSOR_3 A5
#define SENSOR_4 A6

/* GAME STATES */
#define WAITING 0 // done
#define STATE_COLORS 1 // done
#define STATE_LASERS 2 // done
#define STATE_BLINK 3 // done
#define STATE_TANGRAMS 4 // done
#define STATE_STORY 5 // almost done
#define STATE_SCALE 6 
#define STATE_PEOPLE 7 // done
#define STATE_MAZE 8 //
#define WIN 9

/* COLORS */
#define COLOR_OFF 0
#define COLOR_RED 1
#define COLOR_ORANGE 2
#define COLOR_YELLOW 3
#define COLOR_GREEN 4
#define COLOR_BLUE 5
#define COLOR_PURPLE 6
#define COLOR_WHITE 7

/* BUTTONS */
#define R 1
#define L 2
#define Fr 3

/* CORRESPONDING TO AUDIO FILES ON MINI-SD CARD */
/* NAMED THEM 1-9 (like 001.mp3, 002.mp3), BUT FOR SOME REASON HAD TO REFER TO THEM as 2-10??? */
#define FOUR73 2 /*tangrams-red lock*/
#define SEVEN28 3 /*maze-green lock*/
#define TWO90 4 /*weights-silver lock*/
#define THREE61 5 /*people-black lock*/
#define BIGWIN 6
#define DING 7
#define LEFTRIGHT 9 
#define WAMP 8
#define LEFTRIGHTINSTR 10

DFRobotDFPlayerMini myDFPlayer;
// LED Strip - Set the first variable to the NUMBER of pixels. 9 pixels in a row
Adafruit_WS2801 STRIP = Adafruit_WS2801(9, dataPin, clockPin);
HX711 SCALE;

int GAME_STATE;

int COLOR_LEFT;
int COLOR_FRONT;
int COLOR_RIGHT;
int STORY_BUTTONS[14] = {R, L, R, R, L, L, R, L, L, R, R, L, R, L};
int MAX_STORY_BUTTONS = 14; // set to length of list above
int USER_STORY_BUTTONS[17];
int STORY_BUTTON_ATTEMPTS;
float SCALE_FACTOR = 357.1f;
int BUTTON_PUSHED;
/* for checking to see if the button has changed (was pushed on last check, now is not) so we don't assume it's pushed again every time we check */
int front_was;
int right_was;
int left_was;

  void setup() {
    STORY_BUTTON_ATTEMPTS = 0;
    front_was = LOW;
    right_was = LOW;
    left_was = LOW;

    #if defined(__AVR_ATtiny85__) && (F_CPU == 16000000L)
      clock_prescale_set(clock_div_1); // Enable 16 MHz on Trinket
    #endif
  
    Serial.begin(9600);
    myDFPlayer.begin(Serial);

    pinMode(HALL_FRONT, INPUT);
    pinMode(HALL_BACK, INPUT);
    pinMode(BLACK_LIGHT_1, OUTPUT);
    pinMode(BLACK_LIGHT_2, OUTPUT);
    pinMode(BLACK_LIGHT_3, OUTPUT);
    pinMode(LASER_1, OUTPUT);
    pinMode(LASER_2, OUTPUT);
    pinMode(LASER_SENSOR_1, INPUT);
    pinMode(LASER_SENSOR_2, INPUT);
    pinMode(SENSOR_1, INPUT);
    pinMode(SENSOR_2, INPUT);
    pinMode(SENSOR_3, INPUT);
    pinMode(SENSOR_4, INPUT);
    pinMode(BUTTON_FRONT, INPUT);
    pinMode(BUTTON_RIGHT, INPUT);
    pinMode(BUTTON_LEFT, INPUT);
    pinMode(RELAY_LOCK, OUTPUT);
  
    SCALE.begin(DOUT, CLK);
    SCALE.set_scale();
    SCALE.tare();
    
    COLOR_LEFT = COLOR_OFF;
    COLOR_FRONT = COLOR_OFF;
    COLOR_RIGHT = COLOR_OFF;

    STRIP.begin();
    // Update LED contents, to start they are all 'off'; sometimes they'd get stuck in ON states
    for (int i=0;i<9;i++) {
      STRIP.setPixelColor(i, Color(0,0,0));
    }
    STRIP.show();
    // make sure lasers are off too
    digitalWrite(LASER_1, LOW);   
    digitalWrite(LASER_2, LOW);   

    Serial.println("starting");
    /*this 10 sec delay seemed to help make sure the DFPlayer had time to get initialized before asking it to play an audio file*/
    delay (10000);
    GAME_STATE = WAITING;
  }

void loop() {
  Serial.println(GAME_STATE);
  if (GAME_STATE==WAITING) {
    // check to see if BUTTON_RIGHT and BUTTON_LEFT are pressed togther
    int button_right_i = digitalRead(BUTTON_RIGHT);
    int button_left_i = digitalRead(BUTTON_LEFT);

    if (button_left_i == HIGH && left_was == LOW && button_right_i == HIGH && right_was == LOW) {
      Serial.println("you pushed BOTH");
      GAME_STATE = STATE_COLORS;
      myDFPlayer.play (DING);
      
    }
    right_was = button_right_i;
    left_was = button_left_i;   
    delay(1000);
  } 
  else if (GAME_STATE==STATE_COLORS) {
    int button_left_i = digitalRead(BUTTON_LEFT);
    int button_front_i = digitalRead(BUTTON_FRONT);
    int button_right_i = digitalRead(BUTTON_RIGHT);
    if (button_left_i == HIGH and left_was == LOW) {
      COLOR_LEFT = COLOR_LEFT + 1;
      COLOR_LEFT = COLOR_LEFT % 7;
    } 
    if (button_front_i == HIGH and front_was == LOW) {
      COLOR_FRONT = COLOR_FRONT + 1;
      COLOR_FRONT = COLOR_FRONT % 7;
    } 
    if (button_right_i == HIGH and right_was == LOW) {
      COLOR_RIGHT = COLOR_RIGHT + 1;
      COLOR_RIGHT = COLOR_RIGHT % 7;
    } 

    STRIP.setPixelColor(0, Color(255,0,0)); // RED
    setLightColor(1, COLOR_LEFT);
    STRIP.setPixelColor(2, Color(255,230,0)); // YELLOW
    STRIP.setPixelColor(3, Color(255,230,0)); // YELLOW
    setLightColor(4, COLOR_FRONT);
    STRIP.setPixelColor(5, Color(0,0,255)); // BLUE
    STRIP.setPixelColor(6, Color(0,0,255)); // BLUE
    setLightColor(7, COLOR_RIGHT);
    STRIP.setPixelColor(8, Color(255,0,0)); // RED
    STRIP.show();  
    delay(500);
    if (COLOR_LEFT == COLOR_ORANGE && COLOR_FRONT == COLOR_GREEN && COLOR_RIGHT == COLOR_PURPLE) {
      GAME_STATE = STATE_LASERS;
      myDFPlayer.play (DING);
      for (int i=0;i<9;i++) {
        STRIP.setPixelColor(i, Color(0,0,0));
      }
      STRIP.show();
    }
    left_was = button_left_i;
    right_was = button_right_i;
    front_was = button_front_i;
    delay(250);
  }
  else if (GAME_STATE==STATE_LASERS) {
    digitalWrite(LASER_1, HIGH);
    digitalWrite(LASER_2, HIGH);
    int laser_sensor_1_i = analogRead(LASER_SENSOR_1);
    int laser_sensor_2_i = analogRead(LASER_SENSOR_2);
    if (laser_sensor_1_i >=800 && laser_sensor_2_i >=800) {
      digitalWrite(LASER_1, LOW);   
      digitalWrite(LASER_2, LOW);   
      GAME_STATE = STATE_BLINK;
      BUTTON_PUSHED=1;
      myDFPlayer.play (DING);
      COLOR_LEFT = 0;
      COLOR_FRONT = 0;
      COLOR_RIGHT = 0;
      // TURN LIGHTS ON
    }
    delay(2000);
  }
  else if (GAME_STATE==STATE_BLINK) {

    int button_left_i = digitalRead(BUTTON_LEFT);
    int button_front_i = digitalRead(BUTTON_FRONT);
    int button_right_i = digitalRead(BUTTON_RIGHT);
    if (button_left_i == HIGH and left_was == LOW) {
      COLOR_LEFT = COLOR_LEFT + 1;
      COLOR_LEFT = COLOR_LEFT % 10;
      BUTTON_PUSHED = 1;
    } 
    if (button_front_i == HIGH and front_was == LOW) {
      COLOR_FRONT = COLOR_FRONT + 1;
      COLOR_FRONT = COLOR_FRONT % 10;
      BUTTON_PUSHED = 1;
    } 
    if (button_right_i == HIGH and right_was == LOW) {
      COLOR_RIGHT = COLOR_RIGHT + 1;
      COLOR_RIGHT = COLOR_RIGHT % 10;
      BUTTON_PUSHED = 1;
    } 
    int del = 200;
    if (BUTTON_PUSHED == 1) {
      int pixi = 0;
      int cnt = 1;
      for (cnt = 1; cnt<=9; cnt++) {
        for (pixi=0;pixi<9;pixi++) {
          if ((pixi !=1 && pixi !=4 && pixi != 7 && (pixi+1>=cnt))) {
            STRIP.setPixelColor(pixi, Color(255,255,255));        
            STRIP.show();  
          }
          if (COLOR_LEFT>=cnt) {
            STRIP.setPixelColor(1, Color(255,255,255));        
            STRIP.show();         
          }
          if (COLOR_FRONT>=cnt) {
            STRIP.setPixelColor(4, Color(255,255,255));        
            STRIP.show();         
          }
          if (COLOR_RIGHT>=cnt) {
            STRIP.setPixelColor(7, Color(255,255,255));        
            STRIP.show();         
          }
        }
        delay(del);
        for (pixi=0;pixi<9;pixi++) {
          STRIP.setPixelColor(pixi, Color(0,0,0));         
          STRIP.show();  
        }     
        delay(del);
      }
    }    
    if (COLOR_LEFT == 2 && COLOR_FRONT == 5 && COLOR_RIGHT == 8) {
      myDFPlayer.play (DING);
      delay(2000);
      for (int i=0;i<9;i++) {
        STRIP.setPixelColor(i, Color(0,0,0));
      }
      STRIP.show();
      GAME_STATE = STATE_TANGRAMS;     
      myDFPlayer.play (FOUR73);
     
      delay(5000);
    }
    left_was = 0;
    right_was = 0;
    front_was = 0;
    BUTTON_PUSHED = 0;
    delay(300);
  }
  else if (GAME_STATE==STATE_TANGRAMS) {
    digitalWrite(BLACK_LIGHT_1, HIGH);
    digitalWrite(BLACK_LIGHT_2, HIGH);
    digitalWrite(BLACK_LIGHT_3, HIGH);
    int hall_front_i = digitalRead(HALL_FRONT);
    if (hall_front_i == 0) {
      digitalWrite(BLACK_LIGHT_1, LOW);
      digitalWrite(BLACK_LIGHT_2, LOW);
      digitalWrite(BLACK_LIGHT_3, LOW);
      GAME_STATE = STATE_STORY;
      myDFPlayer.play (DING);
      STORY_BUTTON_ATTEMPTS = 0;
      // PLAY STORY LOOP
      delay(2000);
      myDFPlayer.play (LEFTRIGHTINSTR);
      delay(11500);
    }
    
    delay(1500);    
  }
  else if (GAME_STATE==STATE_STORY) {
    int button_front_i = digitalRead(BUTTON_FRONT);
    if (button_front_i == HIGH && front_was == LOW) {
      //PLAY STORY - restarting
      STORY_BUTTON_ATTEMPTS = 0;
      myDFPlayer.play (LEFTRIGHT);
    }
    front_was = button_front_i;
    //LISTEN FOR BUTTONS
    if (STORY_BUTTON_ATTEMPTS >= MAX_STORY_BUTTONS) {
      //compare list
      // next stage if same and also end story loop
      // else blink lights and reset USER_STORY_BUTTONS and STORY_BUTTON_ATTEMPTS
      boolean isright = true;
      for (int r=0;r<STORY_BUTTON_ATTEMPTS;r++) {
        if (USER_STORY_BUTTONS[r] != STORY_BUTTONS[r]) {
          isright = false;
        }
      }
      if (isright == true) {
        GAME_STATE = STATE_SCALE;
        myDFPlayer.play (DING);
        delay(2000);
        myDFPlayer.play (TWO90);
        delay(3000);
      } else {
        STORY_BUTTON_ATTEMPTS = 0;
        myDFPlayer.play (WAMP);        
        delay(3000);
        myDFPlayer.play (LEFTRIGHT);        
      }
    } else {
      int button_right_i = digitalRead(BUTTON_RIGHT);
      int button_left_i = digitalRead(BUTTON_LEFT);
      if (button_left_i == HIGH && left_was == LOW) {
        USER_STORY_BUTTONS[STORY_BUTTON_ATTEMPTS] = L;
        STORY_BUTTON_ATTEMPTS++;
      }
      if (button_right_i == HIGH && right_was == LOW) {
        USER_STORY_BUTTONS[STORY_BUTTON_ATTEMPTS] = R;
        STORY_BUTTON_ATTEMPTS++; 
      }
      right_was = button_right_i;
      left_was = button_left_i;
    }
  }
  else if (GAME_STATE==STATE_SCALE) {
    if (SCALE.is_ready()) {
      int reading = (int)(SCALE.get_units(5));
      reading = reading / SCALE_FACTOR;
      Serial.print("SCALE: ");
      Serial.println(reading);
      // giving a little 4g lee-way here so i don't have to re-calibrate often
      if (abs(reading - 49) < 4) {
        // next state
        GAME_STATE = STATE_PEOPLE;
        myDFPlayer.play (DING);
        delay(2000);
        myDFPlayer.play (THREE61);
        delay(3000);
      }
    } else {
      Serial.println("HX711 NOT FOUND.");
    }
    delay(3000);    
  }
  else if (GAME_STATE==STATE_PEOPLE) {
    int sensor_1_i = analogRead(SENSOR_1);
    int sensor_2_i = analogRead(SENSOR_2);
    int sensor_3_i = analogRead(SENSOR_3);
    int sensor_4_i = analogRead(SENSOR_4);
    Serial.print("SENSOR 1: ");
    Serial.println(sensor_1_i);
    Serial.print("SENSOR 2: ");
    Serial.println(sensor_2_i);
    Serial.print("SENSOR 3: ");
    Serial.println(sensor_3_i);
    Serial.print("SENSOR 4: ");
    Serial.println(sensor_4_i);
    delay(5000);
    if (sensor_3_i < sensor_4_i && sensor_4_i < sensor_1_i && sensor_1_i < sensor_2_i) {
      myDFPlayer.play (DING);
      GAME_STATE = STATE_MAZE;
      delay(2000);
      myDFPlayer.play (SEVEN28);
      delay(3000);    
    }
    delay(2000);
  }
  else if (GAME_STATE==STATE_MAZE) {
    digitalWrite(HALL_BACK, HIGH);
    int hall_back_i = digitalRead(HALL_BACK);
    digitalWrite(HALL_BACK, LOW);
    if (hall_back_i == 0) {
      GAME_STATE = WIN;
    }    
    delay(1000);
  }
  else if (GAME_STATE==WIN) {
    // BLINK WINING LIGHTS    
    digitalWrite(RELAY_LOCK, HIGH);    
    delay(3000);
    digitalWrite(RELAY_LOCK, LOW);    
    winSound();
    winLights();
    GAME_STATE = WAITING;
  }
  
} // END LOOP

void winSound() {
  // play WIN sound so user knows he correctly completed section
  myDFPlayer.play (BIGWIN);
  delay(5000);
}

//lights to show user he has won
//blinks 15 times then stays lit up
void winLights() {
    for (int i=0;i<9;i++) {
      STRIP.setPixelColor(i, Color(0,0,0));
    }
    STRIP.show();
    delay(1000);
  for (int m=0;m<6; m++) {
    for (int i=0;i<9;i++) {
      STRIP.setPixelColor(i, Color(255,255,255));
      STRIP.show();
      delay(200);
    }
    delay(500);
    for (int i=0;i<9;i++) {
      STRIP.setPixelColor(i, Color(0,0,0));
      STRIP.show();
      delay(200);
    }
    delay(500);
  } // end for
}

void setLightColor(int loc, int colour) {
  if (colour == COLOR_OFF) {
    STRIP.setPixelColor(loc, Color(0,0,0)); // OFF    
  } else if (colour == COLOR_RED) {
    STRIP.setPixelColor(loc, Color(255,0,0)); // RED
  } else if (colour == COLOR_ORANGE) {
    STRIP.setPixelColor(loc, Color(255,127,0)); // ORANGE
  } else if (colour == COLOR_YELLOW) {
    STRIP.setPixelColor(loc, Color(255,255,0)); // YELLOW
  } else if (colour == COLOR_GREEN) {
    STRIP.setPixelColor(loc, Color(0,255,0)); // GREEN
  } else if (colour == COLOR_BLUE) {
    STRIP.setPixelColor(loc, Color(0,0,255)); // BLUE
  } else if (colour == COLOR_PURPLE) {
    STRIP.setPixelColor(loc, Color(143,0,255)); // PURPLE 
  } else if (colour == COLOR_WHITE) {
    STRIP.setPixelColor(loc, Color(255,255,255)); // WHITE 
  }
}

// Create a 24 bit color value from R,G,B
uint32_t Color(byte r, byte g, byte b)
{
  uint32_t c;
  c = r;
  c <<= 8;
  c |= g;
  c <<= 8;
  c |= b;
  return c;
}
